@@ -36,7 +36,7 @@ static Bool HandleHextileEncoding32(int x, int y, int w, int h);
 
 int rfbsock;
 char *desktopName;
-rfbPixelFormat myFormat;
+#define myFormat (si.format)
 rfbServerInitMsg si;
 struct timeval updateRequestTime;
 Bool sendUpdateRequest;
@@ -292,6 +292,7 @@ SetFormatAndEncodings()
 
     spf.type = rfbSetPixelFormat;
     spf.format = myFormat;
+
     spf.format.redMax = Swap16IfLE(spf.format.redMax);
     spf.format.greenMax = Swap16IfLE(spf.format.greenMax);
     spf.format.blueMax = Swap16IfLE(spf.format.blueMax);
@@ -306,9 +307,11 @@ SetFormatAndEncodings()
 	encs[se->nEncodings++] = Swap32IfLE(explicitEncodings[i]);
     }
 
+#if 0
     if (SameMachine(rfbsock)) {
 	encs[se->nEncodings++] = Swap32IfLE(rfbEncodingRaw);
     }
+#endif
 
     if (addCopyRect)
 	encs[se->nEncodings++] = Swap32IfLE(rfbEncodingCopyRect);
@@ -424,6 +427,9 @@ HandleRFBServerMessage()
 	int linesToRead;
 	int bytesPerLine;
 	int i;
+#ifdef DEBUG
+	fprintf(stderr,"Framebufferupdate.\n");
+#endif
 
 	if (!ReadExact(rfbsock, ((char *)&msg.fu) + 1,
 			 sz_rfbFramebufferUpdateMsg - 1))
@@ -480,6 +486,9 @@ HandleRFBServerMessage()
 	    case rfbEncodingCopyRect:
 	    {
 		rfbCopyRect cr;
+#ifdef DEBUG
+		fprintf(stderr,"EncodingCopyRect.\n");
+#endif
 
 		if (!ReadExact(rfbsock, (char *)&cr, sz_rfbCopyRect))
 		    return False;
@@ -497,6 +506,10 @@ HandleRFBServerMessage()
 		rfbRectangle subrect;
 		int j;
 
+#ifdef DEBUG
+		fprintf(stderr,"EncodingRRE.\n");
+#endif
+
 		if (!ReadExact(rfbsock, (char *)&hdr, sz_rfbRREHeader))
 		    return False;
 
@@ -565,6 +578,11 @@ HandleRFBServerMessage()
 		CARD8 *ptr;
 		register int x, y, w, h;
 
+#ifdef DEBUG
+		fprintf(stderr,"EncodingCoRRE.\n");
+#endif
+
+
 		if (!ReadExact(rfbsock, (char *)&hdr, sz_rfbRREHeader))
 		    return False;
 
@@ -607,6 +625,9 @@ HandleRFBServerMessage()
 
 	    case rfbEncodingHextile:
 	    {
+#ifdef DEBUG
+		fprintf(stderr,"EncodingHexTile.\n");
+#endif
 		switch (myFormat.bitsPerPixel) {
 		case 8:
 		    if (!HandleHextileEncoding8(rect.r.x, rect.r.y,
@@ -647,30 +668,23 @@ HandleRFBServerMessage()
 
     case rfbServerCutText:
     {
-	char *str;
-        XWindowAttributes   attrs;
+      char *str;
+      extern void handle_cut_text(char*,size_t);
 
-	if (!ReadExact(rfbsock, ((char *)&msg) + 1,
-			 sz_rfbServerCutTextMsg - 1))
-	    return False;
+      if (!ReadExact(rfbsock, ((char *)&msg) + 1,
+		     sz_rfbServerCutTextMsg - 1))
+	return False;
+      
+      msg.sct.length = Swap32IfLE(msg.sct.length);
 
-	msg.sct.length = Swap32IfLE(msg.sct.length);
+      str = malloc(msg.sct.length + 1);
+      if (!ReadExact(rfbsock, str, msg.sct.length))
+	return False;
 
-	str = malloc(msg.sct.length);
-
-	if (!ReadExact(rfbsock, str, msg.sct.length))
-	    return False;
-
-        XGetWindowAttributes(dpy, RootWindow(dpy, 0), &attrs);
-	XSelectInput(dpy, RootWindow(dpy, 0),
-                     attrs.your_event_mask & ~PropertyChangeMask);
-	XStoreBytes(dpy, str, msg.sct.length);
-	XSetSelectionOwner(dpy, XA_PRIMARY, None, CurrentTime);
-	XSelectInput(dpy, RootWindow(dpy, 0), attrs.your_event_mask);
-
-	free(str);
-
-	break;
+      str[msg.sct.length]=0;
+      handle_cut_text(str,  msg.sct.length);
+      
+      break;
     }
 
     default:
