@@ -51,6 +51,11 @@ int edge_width=1;
 int restingx=-1;
 int restingy=-1;
 int emulate_wheel=1;
+int wheel_button_up=4;
+
+char *client_selection_text=0;
+size_t client_selection_text_length=0;
+
 
 /*
  * This variable is true (1) if the mouse is on the same screen as the one
@@ -193,17 +198,6 @@ Bool CreateXWindow(void)
       mouseOnScreen = 1;
     }
 
-    myFormat.bitsPerPixel = 8;
-    myFormat.depth = 8;
-    myFormat.trueColour = 1;
-    myFormat.bigEndian = 0;
-    myFormat.redMax = 7;
-    myFormat.greenMax = 7;
-    myFormat.blueMax = 3;
-    myFormat.redShift = 0;
-    myFormat.greenShift = 3;
-    myFormat.blueShift = 6;
-
     nullPixmap = XCreatePixmap(dpy, DefaultRootWindow(dpy), 1, 1, 1);
     grabCursor = 
       XCreatePixmapCursor(dpy, nullPixmap, nullPixmap,
@@ -229,33 +223,33 @@ ShutdownX()
  * HandleXEvents.
  */
 
-Bool
-HandleXEvents()
+Bool HandleXEvents()
 {
-    XEvent ev;
-
-
-     /* presumably XCheckMaskEvent is more efficient than XCheckIfEvent -GRM */
-/*    while (XCheckIfEvent(dpy, &ev, AllXEventsPredicate, NULL)) {*/
-    while (XCheckMaskEvent(dpy, ~0, &ev)) {
-      if (ev.xany.window == topLevel) {
-
-	    if (!HandleTopLevelEvent(&ev))
-		return False;
-
-	} else if (ev.xany.window == DefaultRootWindow(dpy) ||
-                   ev.xany.window == RootWindow(dpy, 0)) {
-
-	    if (!HandleRootEvent(&ev))
-		return False;
-
-	} else if (ev.type == MappingNotify) {
-
-	    XRefreshKeyboardMapping(&ev.xmapping);
-	}
+  XEvent ev;
+  
+  /* presumably XCheckMaskEvent is more efficient than XCheckIfEvent -GRM */
+    while (XCheckIfEvent(dpy, &ev, AllXEventsPredicate, NULL))
+/*  while (XCheckMaskEvent(dpy, ~0, &ev)) */
+  {
+    if (ev.xany.window == topLevel)
+    {
+      
+      if (!HandleTopLevelEvent(&ev))
+	return False;
+      
     }
+    else if (ev.xany.window == DefaultRootWindow(dpy) ||
+	     ev.xany.window == RootWindow(dpy, 0)) {
+      if (!HandleRootEvent(&ev))
+	return False;
+    }
+    else if (ev.type == MappingNotify)
+    {
+      XRefreshKeyboardMapping(&ev.xmapping);
+    }
+  }
 
-    return True;
+  return True;
 }
 
 #define EW (edge == EDGE_EAST || edge==EDGE_WEST)
@@ -281,10 +275,44 @@ static int leave_translate(int isedge, int width, int pos)
 #define EDGE(X) ((X)?edge_width:0)
 
 #define SCALEX(X) \
-( ((X) - EDGE(edge==EDGE_WEST))*si.framebufferWidth/(displayWidth - EDGE(EW)) )
+( ((X)-EDGE(edge==EDGE_WEST ))*(si.framebufferWidth-1 )/(displayWidth -EDGE(EW)) )
 
 #define SCALEY(Y) \
-( ((Y)-EDGE(edge==EDGE_NORTH))*si.framebufferHeight/(displayHeight-EDGE(NS)) )
+( ((Y)-EDGE(edge==EDGE_NORTH))*(si.framebufferHeight-1)/(displayHeight-EDGE(NS)) )
+
+
+static void grabit(XEvent *ev)
+{
+  Window selection_owner;
+
+  XGrabPointer(dpy, topLevel, True,
+	       PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
+	       GrabModeAsync, GrabModeAsync,
+	       None, grabCursor, CurrentTime);
+  XGrabKeyboard(dpy, topLevel, True, 
+		GrabModeAsync, GrabModeAsync,
+		CurrentTime);
+  XFlush(dpy);
+  
+  XWarpPointer(dpy,None, DefaultRootWindow(dpy),0,0,0,0,
+	       enter_translate(EW,displayWidth ,ev->xcrossing.x_root),
+	       enter_translate(NS,displayHeight,ev->xcrossing.y_root));
+  
+  SendPointerEvent(SCALEX(ev->xcrossing.x_root), SCALEY(ev->xcrossing.y_root),
+		   (ev->xcrossing.state & 0x1f00) >> 8);
+  grabbed=1;
+  mouseOnScreen = 1;
+
+  selection_owner=XGetSelectionOwner(dpy, XA_PRIMARY);
+/*   fprintf(stderr,"Selection owner: %lx\n",(long)selection_owner); */
+
+  if(selection_owner != None && selection_owner != topLevel)
+  {
+    XConvertSelection(dpy,
+		      XA_PRIMARY, XA_STRING, XA_CUT_BUFFER0,
+		      topLevel, CurrentTime);
+  }
+}
 
 /*
  * HandleTopLevelEvent.
@@ -298,8 +326,8 @@ static void shortsleep(int usec)
   select(0,0,0,0,&timeout);
 }
 
-static Bool
-HandleTopLevelEvent(XEvent *ev)
+
+static Bool HandleTopLevelEvent(XEvent *ev)
 {
   Bool grab;
   int i;
@@ -308,7 +336,85 @@ HandleTopLevelEvent(XEvent *ev)
   int buttonMask;
   KeySym ks;
   
-  switch (ev->type) {
+  switch (ev->type)
+  {
+    case SelectionRequest:
+    {
+      XEvent reply;
+
+      XSelectionRequestEvent *req=& ev->xselectionrequest;
+
+      reply.xselection.type=SelectionNotify;
+      reply.xselection.display=req->display;
+      reply.xselection.selection=req->selection;
+      reply.xselection.requestor=req->requestor;
+      reply.xselection.target=req->target;
+      reply.xselection.time=req->time;
+      reply.xselection.property=None;
+      
+      if(client_selection_text)
+      {
+	if(req->target == XA_STRING)
+	{
+	  int ret;
+	  if(req->property == None)
+	    req->property = XA_CUT_BUFFER0;
+
+#if 0
+	  fprintf(stderr,"Setting property %d on window %x to: %s (len=%d)\n",req->property, req->requestor, client_selection_text,client_selection_text_length);
+#endif
+
+	  XChangeProperty(dpy,
+			  req->requestor,
+			  req->property,
+			  XA_STRING,
+			  8,
+			  PropModeReplace,
+			  client_selection_text,
+			  client_selection_text_length);
+
+	  reply.xselection.property=req->property;
+	  
+	}
+      }
+      XSendEvent(dpy, req->requestor, 0,0, &reply);
+      XFlush (dpy);
+    }
+    return 1;
+
+    case SelectionNotify:
+    {
+      Atom t;
+      unsigned long len=0;
+      unsigned long bytes_left=0;
+      int format;
+      unsigned char *data=0;
+      int ret;
+
+      ret=XGetWindowProperty(dpy,
+			     topLevel,
+			     XA_CUT_BUFFER0,
+			     0,
+			     0x100000,
+			     1, /* delete */
+			     XA_STRING,
+			     &t,
+			     &format,
+			     &len,
+			     &bytes_left,
+			     &data);
+
+      if(format == 8)
+	SendClientCutText(data, len);
+#ifdef DEBUG
+      fprintf(stderr,"GOT selection info: ret=%d type=%d fmt=%d len=%d bytes_left=%d %p '%s'\n",
+	ret, (int)t,format,len,bytes_left,data,data);
+#endif
+      if(data) XFree(data);
+			 
+    }
+    return 1;
+      
     case VisibilityNotify:
       /*
        * I avoid resurfacing when the window becomes fully obscured, because
@@ -338,23 +444,7 @@ HandleTopLevelEvent(XEvent *ev)
     case EnterNotify:
       if(!grabbed && ev->xcrossing.mode==NotifyNormal)
       {
-	XGrabPointer(dpy, topLevel, True,
-		     PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
-		     GrabModeAsync, GrabModeAsync,
-		     None, grabCursor, CurrentTime);
-	XGrabKeyboard(dpy, topLevel, True, 
-		      GrabModeAsync, GrabModeAsync,
-		      CurrentTime);
-	XFlush(dpy);
-	XWarpPointer(dpy,None, DefaultRootWindow(dpy),0,0,0,0,
-		     enter_translate(EW,displayWidth ,ev->xcrossing.x_root),
-		     enter_translate(NS,displayHeight,ev->xcrossing.y_root));
-        mouseOnScreen = 1;
-	
-	SendPointerEvent(SCALEX(ev->xcrossing.x_root),
-			 SCALEY(ev->xcrossing.y_root),
-			 (ev->xmotion.state & 0x1f00) >> 8);
-	grabbed=1;
+	grabit(ev);
       }
       return 1;
       
@@ -383,26 +473,27 @@ HandleTopLevelEvent(XEvent *ev)
         /* First, check for normal edges (applies to single screen or
          * screen edge that doesn't switch screens)
          */
-        if (ev->xmotion.same_screen) {
+        if (ev->xmotion.same_screen)
+	{
           x = ev->xmotion.x_root;
           y = ev->xmotion.y_root;
           warpWindow = DefaultRootWindow(dpy);
 	  switch(edge)
 	  {
 	    case EDGE_NORTH: 
-              d=ev->xmotion.y_root == displayHeight-1;
+              d=y >= displayHeight-edge_width;
               y = edge_width;
               break;
 	    case EDGE_SOUTH:
-              d=ev->xmotion.y_root == 0;
+              d=y < edge_width;
               y = displayHeight - edge_width -1;
               break;
 	    case EDGE_EAST:
-              d=ev->xmotion.x_root == 0;
+              d=x < edge_width;
               x = displayWidth -  edge_width -1;
               break;
 	    case EDGE_WEST:
-              d=ev->xmotion.x_root == displayWidth-1;
+              d=x >= displayWidth-edge_width;
               x = edge_width;
               break;
 	  }
@@ -457,7 +548,8 @@ HandleTopLevelEvent(XEvent *ev)
           mouseOnScreen = warpWindow == DefaultRootWindow(dpy);
 	  XFlush(dpy);
 	  
-	  for (i = 255; i >= 0; i--) {
+	  for (i = 255; i >= 0; i--)
+	  {
 	    if (modifierPressed[i]) {
 	      if (!SendKeyEvent(XKeycodeToKeysym(dpy, i, 0), False))
 		return False;
@@ -475,10 +567,10 @@ HandleTopLevelEvent(XEvent *ev)
     case ButtonRelease:
       if (ev->xbutton.button > 3 && emulate_wheel)
       {
-	if (ev->xbutton.button == 4)
-	  ks = XK_Down;
-	else
+	if (ev->xbutton.button == wheel_button_up)
 	  ks = XK_Up;
+	else
+	  ks = XK_Down;
 
 	if (ev->type == ButtonPress)
 	  SendKeyEvent(ks, 1); /* keydown */
@@ -514,8 +606,8 @@ HandleTopLevelEvent(XEvent *ev)
 	/* This fixes the 'shift-tab' problem - Hubbe */
 	switch(ks)
 	{
-#if XK_ISO_Left_Tab != 0x10000ff74
-	  case 0x10000ff74: /* HP-UX */
+#if XK_ISO_Left_Tab != 0x1000ff74
+	  case 0x1000ff74: /* HP-UX */
 #endif
 	  case XK_ISO_Left_Tab: ks=XK_Tab; break;
 	}
@@ -544,19 +636,18 @@ HandleTopLevelEvent(XEvent *ev)
  * HandleRootEvent.
  */
 
-static Bool
-HandleRootEvent(XEvent *ev)
+static Bool HandleRootEvent(XEvent *ev)
 {
-    char *str;
-    int len;
-
-    Bool nowOnScreen;
-    Bool grab;
-    
-    int x, y;
-
-    switch (ev->type) {
-
+  char *str;
+  int len;
+  
+  Bool nowOnScreen;
+  Bool grab;
+  
+  int x, y;
+  
+  switch (ev->type)
+  {
     case EnterNotify:
     case LeaveNotify:
       /*
@@ -565,91 +656,99 @@ HandleRootEvent(XEvent *ev)
        */
       if (ev->xcrossing.subwindow == topLevel &&
           !ev->xcrossing.same_screen) {
-          break;
+	break;
       }
-
+      
       grab = 0;
-      if(!grabbed) {
-          nowOnScreen =  ev->xcrossing.same_screen;
-          if (mouseOnScreen != nowOnScreen) {
-              /*
-               * Mouse has left, or entered, the screen. We must grab if
-               * the mouse is now near the edge we're watching.
-               * 
-               * If we do grab, the mouse coordinates are left alone.
-               * The test, however, depends on whether the mouse entered
-               * or left the screen.
-               * 
-               * - GRM
-               */
-              x = ev->xcrossing.x_root;
-              y = ev->xcrossing.x_root;
-              if (!nowOnScreen) {
-                  x = enter_translate(EW,displayWidth,ev->xcrossing.x_root);
-                  y = enter_translate(NS,displayHeight,ev->xcrossing.y_root);
-              }
-	      switch(edge)
-	      {
-	        case EDGE_NORTH:
-                  grab=y < displayHeight / 2;
-                  break;
-	        case EDGE_SOUTH:
-                  grab=y > displayHeight / 2;
-                  break;
-	        case EDGE_EAST:
-                  grab=x > displayWidth / 2;
-                  break;
-	        case EDGE_WEST:
-                  grab=x < displayWidth / 2;
-                  break;
-              }
-          }
-          mouseOnScreen = nowOnScreen;
+      if(!grabbed)
+      {
+	nowOnScreen =  ev->xcrossing.same_screen;
+	if (mouseOnScreen != nowOnScreen) {
+	  /*
+	   * Mouse has left, or entered, the screen. We must grab if
+	   * the mouse is now near the edge we're watching.
+	   * 
+	   * If we do grab, the mouse coordinates are left alone.
+	   * The test, however, depends on whether the mouse entered
+	   * or left the screen.
+	   * 
+	   * - GRM
+	   */
+	  x = ev->xcrossing.x_root;
+	  y = ev->xcrossing.x_root;
+	  if (!nowOnScreen) {
+	    x = enter_translate(EW,displayWidth,ev->xcrossing.x_root);
+	    y = enter_translate(NS,displayHeight,ev->xcrossing.y_root);
+	  }
+	  switch(edge)
+	  {
+	    case EDGE_NORTH:
+	      grab=y < displayHeight / 2;
+	      break;
+	    case EDGE_SOUTH:
+	      grab=y > displayHeight / 2;
+	      break;
+	    case EDGE_EAST:
+	      grab=x > displayWidth / 2;
+	      break;
+	    case EDGE_WEST:
+	      grab=x < displayWidth / 2;
+	      break;
+	  }
+	}
+	mouseOnScreen = nowOnScreen;
       }
-
-        /*
-         * Do not grab if this is the result of an ungrab
-         * or grab (caused by us, usually).
-         * 
-         * - GRM
-         */
+      
+      /*
+       * Do not grab if this is the result of an ungrab
+       * or grab (caused by us, usually).
+       * 
+       * - GRM
+       */
       if(grab && ev->xcrossing.mode == NotifyNormal)
       {
-	XWarpPointer(dpy,None, DefaultRootWindow(dpy),0,0,0,0, 
-                     ev->xcrossing.x_root, ev->xcrossing.y_root);
-	XGrabPointer(dpy, topLevel, True,
-		     PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
-		     GrabModeAsync, GrabModeAsync,
-		     None, grabCursor, CurrentTime);
-	XGrabKeyboard(dpy, topLevel, True, 
-		      GrabModeAsync, GrabModeAsync,
-		      CurrentTime);
-	XFlush(dpy);
-	     
-	
-	SendPointerEvent(SCALEX(ev->xcrossing.x_root), SCALEY(ev->xcrossing.y_root),
-			 (ev->xcrossing.state & 0x1f00) >> 8);
-	grabbed=1;
-        mouseOnScreen = 1;
+	grabit(ev);
       }
       break;
-
+      
     case PropertyNotify:
-	if (ev->xproperty.atom == XA_CUT_BUFFER0) {
-
-	    str = XFetchBytes(dpy, &len);
-	    if (str) {
-		if (!SendClientCutText(str, len))
-		    return False;
-		XFree(str);
-	    }
+      if (ev->xproperty.atom == XA_CUT_BUFFER0)
+      {
+	str = XFetchBytes(dpy, &len);
+	if (str) {
+#ifdef DEBUG
+	  fprintf(stderr,"GOT CUT TEXT: %s\n",str);
+#endif
+	  if (!SendClientCutText(str, len))
+	    return False;
+	  XFree(str);
 	}
-	break;
-    }
-
-    return True;
+      }
+      break;
+  }
+  
+  return True;
 }
 
+void handle_cut_text(char *str, size_t len)
+{
+  XWindowAttributes   attrs;
+  
+  if(client_selection_text)
+    free((char *)client_selection_text);
+  
+  client_selection_text_length=len;
+  client_selection_text = str;
+  
+  XGetWindowAttributes(dpy, RootWindow(dpy, 0), &attrs);
+  XSelectInput(dpy, RootWindow(dpy, 0),
+	       attrs.your_event_mask & ~PropertyChangeMask);
+  XStoreBytes(dpy, str, len);
+  XSetSelectionOwner(dpy, XA_PRIMARY, topLevel, CurrentTime);
+  XSelectInput(dpy, RootWindow(dpy, 0), attrs.your_event_mask);
+}
+
+
 
 /*
  * AllXEventsPredicate is needed to make XCheckIfEvent return all events.
@@ -658,7 +757,7 @@ HandleRootEvent(XEvent *ev)
 Bool
 AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
 {
-    return True;
+  return True;
 }
 
 
